from typing import Optional

from fastapi import APIRouter, Depends
from redis.asyncio.client import Redis
from starlette.requests import Request

from narq import Narq, constants
from narq.server import templates
from narq.server.depends import get_narq, get_pager, get_redis
from narq.server.models import JobResult
from narq.server.responses import JobResultListOut

router = APIRouter()


@router.get("/data", response_model=JobResultListOut, name="narq.get_results")
async def get_results(
    task: Optional[str] = None,
    job_id: Optional[str] = None,
    start_time: Optional[str] = None,
    end_time: Optional[str] = None,
    worker: Optional[str] = None,
    success: Optional[str] = None,
    pager=Depends(get_pager),
):
    qs = JobResult.all().select_related("job")
    if task:
        qs = qs.filter(job__task=task)
    if job_id:
        qs = qs.filter(job__job_id=job_id)
    if start_time:
        qs = qs.filter(start_time__gte=start_time)
    if end_time:
        qs = qs.filter(start_time__lte=end_time)
    if worker:
        qs = qs.filter(worker=worker)
    if success:
        qs = qs.filter(success=success == "1")
    results = await qs.limit(pager[0]).offset(pager[1])
    return {"rows": results, "total": await qs.count()}


@router.delete("", name="narq.delete_result")
async def delete_result(ids: str):
    return await JobResult.filter(id__in=ids.split(",")).delete()


@router.get("", include_in_schema=False, name="narq.job_result_page")
async def job_result_page(
    request: Request,
    redis: Redis = Depends(get_redis),
    narq: Narq = Depends(get_narq),
):
    workers_info = await redis.hgetall(constants.WORKER_KEY)

    return templates.TemplateResponse(
        "result.html",
        {
            "request": request,
            "page_title": "result",
            "tasks": narq.task_map.keys(),
            "workers": workers_info.keys(),
        },
    )
